<?php
/**
 * Api module for querying message group stats.
 *
 * @file
 * @author Tim Gerundt
 * @author Niklas Laxström
 * @copyright Copyright © 2012-2013, Tim Gerundt
 * @license GPL-2.0+
 */

/**
 * Api module for querying message group stats.
 *
 * @ingroup API TranslateAPI
 */
class ApiQueryMessageGroupStats extends ApiStatsQuery {
	public function __construct( $query, $moduleName ) {
		parent::__construct( $query, $moduleName, 'mgs' );
	}

	protected function getData() {
		$params = $this->extractRequestParams();
		$group = MessageGroups::getGroup( $params['group'] );
		if ( !$group ) {
			if ( method_exists( $this, 'dieWithError' ) ) {
				$this->dieWithError( array( 'apierror-missingparam', 'mgsgroup' ) );
			} else {
				$this->dieUsageMsg( array( 'missingparam', 'mgsgroup' ) );
			}
		} elseif ( MessageGroups::isDynamic( $group ) ) {
			if ( method_exists( $this, 'dieWithError' ) ) {
				$this->dieWithError( 'apierror-translate-nodynamicgroups', 'invalidparam' );
			} else {
				$this->dieUsage( 'Dynamic message groups are not supported here', 'invalidparam' );
			}
		}

		return MessageGroupStats::forGroup( $group->getId() );
	}

	protected function makeItem( $item, $stats ) {
		$data = parent::makeItem( $item, $stats );
		$data['code'] = $item; // For BC
		$data['language'] = $item;

		return $data;
	}

	public function getAllowedParams() {
		$params = parent::getAllowedParams();
		$params['group'] = array(
			ApiBase::PARAM_TYPE => 'string',
			ApiBase::PARAM_REQUIRED => true,
		);

		return $params;
	}

	protected function getExamplesMessages() {
		return array(
			'action=query&meta=messagegroupstats&mgsgroup=page-Example'
				=> 'apihelp-query+messagegroupstats-example-1',
		);
	}
}
